#!/bin/sh
#------------------------------------------------------------------
# linux_gos_timeout-install.sh
#
# Author:  NetApp, Inc.
# Date:    11-JUN-2008
# Release: 5.2.111.7428
# Purpose: This script will create a udev rule on a Linux Guest OS
#          which will set the required SCSI I/O timeout (180s) for
#          connectivity to IBM N series storage systems.
#
# Note:    This script is required to be run by root.
#
# Copyright (c) 2008-2009 NetApp, Inc.
# All rights reserved.
#-------------------------------------------------------------------
#Declare global variables
IO_TIMEOUT=190
IO_TMO_FROM_VMWARE_TOOLS=180

echo "Setting IBM N series storage system timeouts in your Linux guest"
# Ensure the user is running as root
USER=`/usr/bin/whoami`

if [ "$USER" != "root" ]
then
   echo ""
   echo "This program must be run as root."
   echo "Exiting."
   echo ""
   exit 2
fi

function trap_abort
{
   Sig handler.
   echo ""
   echo "Execution aborted!"
   echo ""
   cleanup
   exit 1
}

################################################
#Cleans up any of the files used to set the    #
#udev rules.                                   #
################################################
function cleanup
{
   rm -Rf ${PATCH_FILE};
   rm -Rf ${UDEV_RULES}.orig ${UDEV_RULES}.rej
}

##################################################
#This function will verify that the timeout was  #
#correctly set.                                  #
##################################################
function verify_settings
{
   # Verify the timeout setting
   for device_dir in `ls -d /sys/block/sd*`
   do
      device_name=`basename ${device_dir}`
      io_tmo=`cat /sys/block/\${device_name}/device/timeout`
      echo -n "Setting I/O Timeout for /dev/${device_name}"
      if  ( [ ${IO_TIMEOUT} == ${io_tmo} ] || [ ${IO_TMO_FROM_VMWARE_TOOLS} == ${io_tmo} ] ); then
         echo " - SUCCESS!"
      else
         echo " - FAILED"
         RC=3
      fi
   done
}

#####################################################
#Function used to restart the UDEV service after the#
#timeouts have been set.                            #
#####################################################
function restart_udev
{
   # Restart udev to activate the new rules
   if [[ -e /sbin/udevstart ]]; then
      echo ""
      echo "Restarting udev... this may take a few seconds."
      echo ""
      /sbin/udevstart
      sleep 5
      verify_settings
   elif [[ -e /sbin/start_udev ]]; then
      echo ""
      echo "Restarting udev... this may take a few seconds."
      echo ""
      /sbin/start_udev
      sleep 5
      verify_settings
   elif [ "$ven" = "SUSE" ] && ( [ "$osver" = "10" ] || [ "$osver" = "11" ] ); then
      echo""
      echo "Restarting udev ...this may take a few seconds."
      echo ""
      /etc/init.d/boot.udev force-reload
      sleep 5
   else
      echo ""
      echo "Could not determine the correct command to restart udev."
      echo "You must reboot for the changes to take effect."
      echo ""
      RC=1
   fi
}

####################################################
#Generate the udev path file .diff so that the udev#
#rule can be updated. This function will set the   #
#rule based on the version of the Linux guest.     #
####################################################
function generate_patch_diff
{
#if [ "$ven" = "Redhat" ] && [ "$osver" = "4" ]; then
if [ "$ven" = "RedHat" ] && [ "$rver" = "5" ]; then
# Create the shell script that will actually modify the I/O
# Timeout setting
cat >> ${PATCH_FILE} <<-EOFA
--- 50-udev.rules	2008-02-14 14:55:57.000000000 -0500
+++ new-50-udev.rules	2008-02-14 15:26:22.000000000 -0500
@@ -320,7 +320,7 @@
 # st/osst:      1 TYPE_TAPE
 # sg: 		8 changer, [36] scanner
 ACTION=="add", SUBSYSTEM=="scsi" , SYSFS{type}=="0|7|14", \\
-	RUN+="/bin/sh -c 'echo 60 > /sys\$\$DEVPATH/timeout'"
+	RUN+="/bin/sh -c 'echo ${IO_TIMEOUT} > /sys\$\$DEVPATH/timeout'"
 ACTION=="add", SUBSYSTEM=="scsi" , SYSFS{type}=="1", \\
 	RUN+="/bin/sh -c 'echo 900 > /sys\$\$DEVPATH/timeout'" 
EOFA

# Ensure the script is readable
chmod 644 ${PATCH_FILE}
elif [ "$ven" = "SUSE" ] && [ "$osver" = "9" ]; then
# Create the udev rule that will run the io timeout shell script for every scsi device attached. 
cat >> ${PATCH_FILE} <<-EOFB
--- udev.rules  Wed Jun  4 19:23:41 2008
+++ udev.rules.new   Wed Jun  4 19:23:33 2008
@@ -8,6 +8,8 @@
 # external helpers should match /sbin/udev.*.sh
 # they should only use /lib/klibc/bin/ binaries
 #
+# I/O Timeout setting required for connectivity to IBM N series storage systems
+BUS="scsi", PROGRAM="/sbin/udev.scsi_disktimeout.sh"
 BUS="scsi", PROGRAM="/sbin/udev.get_persistent_device_name.sh", NAME="%k" SYMLINK="%c{1+}"
 BUS="usb", PROGRAM="/sbin/udev.get_persistent_device_name.sh", NAME="%k" SYMLINK="%c{1+}"
 BUS="ide", PROGRAM="/sbin/udev.get_persistent_device_name.sh", NAME="%k" SYMLINK="%c{1+}"
EOFB

# Ensure the script is readable
chmod 644 ${PATCH_FILE};
elif [ "$ven" = "SUSE" ] && [ "$osver" = "10" ]; then
	
#create the shell script that will actually modify the I/O
# Timeout setting
cat >> ${PATCH_FILE} <<-EOFA
--- 50-udev-default.rules       Fri Feb 15 10:13:05 2008
+++ new-50-udev-default.rules   Fri Feb 15 10:14:15 2008
@@ -113,7 +113,7 @@
 # sd:		0 TYPE_DISK, 7 TYPE_MOD, 14 TYPE_RBC
 # sr:		4 TYPE_WORM, 5 TYPE_ROM
 # st/osst:	1 TYPE_TAPE
-ACTION=="add", SUBSYSTEM=="scsi" , SYSFS{type}=="0|7|14", RUN+="/bin/sh -c 'echo 60 > /sys\$\$DEVPATH/timeout'"
+ACTION=="add", SUBSYSTEM=="scsi" , SYSFS{type}=="0|7|14", RUN+="/bin/sh -c 'echo ${IO_TIMEOUT} > /sys\$\$DEVPATH/timeout'"
 ACTION=="add", SUBSYSTEM=="scsi" , SYSFS{type}=="1", RUN+="/bin/sh -c 'echo 900 > /sys\$\$DEVPATH/timeout'"
 SUBSYSTEM=="scsi_device", ACTION=="add", SYSFS{type}=="0|7|14", RUN+="/sbin/modprobe sd_mod"
 SUBSYSTEM=="scsi_device", ACTION=="add", SYSFS{type}=="1", SYSFS{vendor}=="On[sS]tream", RUN+="/sbin/modprobe osst"
EOFA

# Ensure the script is readable
chmod 644 ${PATCH_FILE}
fi
}

#########################################################
#This function will create a brand new rules file on    #
#systems where there is no need to modify the existing  #
#files. Currently, this function (and UDEV format)      #
#is used only for SLES11 type guests. All other types   #
#of linux guests use other functions.                   #
#########################################################
function create_sles11_rule_file
{

if [ "$ven" = "SUSE" ] && [ "$osver" = "11" ]; then

    # Make sure that the directory exists as expected
    if [ -e ${RULE_DIR} ]; then  
 
        # Create the rule file
        cat > $UDEV_RULE_FILE<<-EOFA 
# I/O Timeout setting required for connectivity to IBM N series storage systems
DRIVERS=="sd", SYSFS{type}=="0|7|14", RUN+="/bin/sh -c 'echo ${IO_TIMEOUT} > /sys\$\$DEVPATH/timeout'"
EOFA
        chmod 644 ${UDEV_RULE_FILE}

    else
        echo "The directory ${RULE_DIR} was not found"
        exit 2
    fi

else
    echo "Creation of independent rule files is not supported on this OS."
    exit 2    
fi
}

#########################################################
#This function will look and see if the timeouts are    #
#set. It will run based on the version of the linux     #
#guest discovered.                                      #
######################################################### 
function check_existing_install
{
#This is to check if the timeout is set for rhel 4 if so it will
#allow you to reinstall it.
if [ "$ven" = "RedHat" ] && [ "$osver" = "4" ]; then
   if [ -e ${UDEV_RULE_FILE} ] && [ -e ${UDEV_SCRIPT} ]; then
      # Prompt to reinstall?
      echo ""
      echo "It appears the settings have already been made."
      echo ""
      until [ "$REINSTALL" = "yes" ] || [ "$REINSTALL" = "no" ]
      do
         echo -n "Would you like to reinstall (yes or no)?> "
         read RESPONSE
         # convert RESPONSE to lowercase
         REINSTALL=`echo $RESPONSE |  tr '[:upper:]' '[:lower:]'`
      done
      if [ "$REINSTALL" = "yes" ]; then
         rm -Rf ${UDEV_SCRIPT} ${UDEV_RULE_FILE}
         # Let the script continue to let is reinstall
      else
         verify_settings
         exit ${RC}
      fi
   fi
#This will check the status of the udev timeout on a sles 9 system
elif [ "$ven" = "SUSE" ] && [ "$osver" = "9" ]; then
   grep "${UDEV_SCRIPT}" "${UDEV_RULES}" > /dev/null 2>&1
   if [[ $? -eq 0 ]] && [ -e ${UDEV_SCRIPT} ]; then
      # Prompt to reinstall?  
      echo ""
      echo "It appears this script has already been run once."
      echo ""
      until [ "$REINSTALL" = "yes" ] || [ "$REINSTALL" = "no" ]
      do
         echo -n "Would you like to reinstall (yes or no)?> "
         read RESPONSE
         # convert RESPONSE to lowercase
         REINSTALL=`echo $RESPONSE |  tr '[:upper:]' '[:lower:]'`
      done
      if [ "$REINSTALL" = "yes" ]; then
         rm -Rf ${UDEV_SCRIPT}
         generate_patch_diff
         # Reverse the Patch to remove it
         patch -Rs ${UDEV_RULES} ${PATCH_FILE}
         cleanup
         # Let the script continue to let is reinstall
      else
         verify_settings
         exit ${RC}
      fi
   fi
#This will check the status of the udev timeout on a sles 11 system
elif [ "$ven" = "SUSE" ] && [ "$osver" = "11" ]; then
   if [ -e ${UDEV_RULE_FILE} ]; then
      # Prompt to reinstall?  
      echo ""
      echo "It appears this script has already been run once."
      echo ""
      until [ "$REINSTALL" = "yes" ] || [ "$REINSTALL" = "no" ]
      do
         echo -n "Would you like to reinstall (yes or no)?> "
         read RESPONSE
         # convert RESPONSE to lowercase
         REINSTALL=`echo $RESPONSE |  tr '[:upper:]' '[:lower:]'`
      done
      if [ "$REINSTALL" = "yes" ]; then
         `rm -Rf ${UDEV_SCRIPT}`
         create_sles11_rule_file
         # Remove the rules file from the directory
         `rm -rf $RULES_FILE`
         # Let the script continue to let is reinstall
      else
         verify_settings
         exit ${RC}
      fi
   fi
#This is the catch all for Rhel5 and Sles 10 since they are similar.
else 
   grep "timeout" "${UDEV_RULES}" | grep -q ${IO_TIMEOUT}
   if [[ $? -eq 0 ]]; then
      # Prompt to reinstall?  
      echo ""
      echo "It appears this script has already been run once."
      echo ""
      until [ "$REINSTALL" = "yes" ] || [ "$REINSTALL" = "no" ]
      do
         echo -n "Would you like to reinstall (yes or no)?> "
         read RESPONSE
         # convert RESPONSE to lowercase
         REINSTALL=`echo $RESPONSE |  tr '[:upper:]' '[:lower:]'`
      done
      if [ "$REINSTALL" = "yes" ]; then
         generate_patch_diff
         # Reverse the Patch to remove it
         patch -Rs ${UDEV_RULES} ${PATCH_FILE}
	 cleanup
         # Let the script continue to let is reinstall
      else
         verify_settings
         exit ${RC}
      fi
   fi
fi
}

#####################################################################
#OS specifiv functions. These contain all fo the above functions and# 
#a few extra things for the particular os.                          # 
#####################################################################
function rhel4
{
echo ""
echo "Found a RedHat Enterprise 4 Guest"

UDEV_RULE_FILE=/etc/udev/rules.d/52-netapp.rules
REINSTALL=""
RC=0
if [ -d /etc/udev/scripts ];then
   UDEV_SCRIPT=/etc/udev/scripts/scsi-disktimeout.sh
else
   UDEV_SCRIPT=/sbin/udev.scsi-disktimeout.sh
fi
trap 'trap_abort' HUP INT QUIT TERM

check_existing_install

# Create the shell script that will actually modify the I/O
# Timeout setting

cat > ${UDEV_SCRIPT} <<-EOFA
#!/bin/bash

for device_dir in \`ls -d /sys/block/sd*\`
do
   device_name=\`basename \${device_dir}\`
   echo "${IO_TIMEOUT}" > /sys/block/\${device_name}/device/timeout
done
EOFA

# Ensure the script is executable
chmod 755 ${UDEV_SCRIPT}

cat > ${UDEV_RULE_FILE} <<-EOFB
# I/O Timeout setting required for connectivity to IBM N series storage systems
KERNEL="sd*[a-z]", BUS="scsi", PROGRAM="${UDEV_SCRIPT}"
EOFB

restart_udev

exit ${RC}

}

################################################################
function sles9
{
echo ""
echo "Found a SuSE Enterprise Linux 9 Guest"
UDEV_SCRIPT=/sbin/udev.scsi_disktimeout.sh
UDEV_RULES=/etc/udev/udev.rules
PATCH_FILE=/etc/udev/sles9_io_timeout.diff
REINSTALL=""
RC=0

trap 'trap_abort' HUP INT QUIT TERM

check_existing_install

# udev requires all binaries used in the rules to be defined in the /lib/klibc/bin folder.
cp /bin/ls  /lib/klibc/bin/ls
cp /bin/echo  /lib/klibc/bin/echo
cp /bin/basename  /lib/klibc/bin/basename

cat > ${UDEV_SCRIPT} <<-EOFA
#!/lib/klibc/bin/sh

for device_dir in \`ls -d /sys/block/sd*\`
do
  device_name=\`basename \${device_dir}\`
  echo "${IO_TIMEOUT}" > /sys/block/\${device_name}/device/timeout
done
EOFA

# Ensure the script is executable
chmod 755 ${UDEV_SCRIPT};

generate_patch_diff

patch -u ${UDEV_RULES} ${PATCH_FILE}

restart_udev

cleanup
exit ${RC}
}

####################################################################
function rhel5
{
echo ""
echo "Found a RedHat Enterprise Linux 5 guest"
PATCH_FILE=/etc/udev/rules.d/rhel5_io_timeout.diff
UDEV_RULES=/etc/udev/rules.d/50-udev.rules
RC=0
trap 'trap_abort' HUP INT QUIT TERM

check_existing_install

generate_patch_diff

patch -u ${UDEV_RULES} ${PATCH_FILE}

restart_udev

#verify_settings

cleanup

exit ${RC}
}

####################################################################
function sles10
{
echo ""
echo "Found a SuSE Enterprise Linux 10 guest"
PATCH_FILE=/etc/udev/rules.d/sles10_io_timeout.diff
UDEV_RULES=/etc/udev/rules.d/50-udev-default.rules
RC=0

trap 'trap_abort' HUP INT QUIT TERM

check_existing_install

generate_patch_diff

patch -u ${UDEV_RULES} ${PATCH_FILE}

restart_udev

verify_settings

cleanup

exit ${RC}
}

####################################################################
function sles11
{
echo ""
echo "Found a SuSE Enterprise Linux 11 guest"
RC=0

trap 'trap_abort' HUP INT QUIT TERM

RULE_DIR=/etc/udev/rules.d
RULE_NAME=89-DataONTAP-scsi_sd.rules
UDEV_RULE_FILE=${RULE_DIR}/${RULE_NAME}

check_existing_install

create_sles11_rule_file

restart_udev

verify_settings

# No cleanup is required on SLES11

exit ${RC}
}

########################################################
#Main                                                  #
#This is where the GOS type is discovered. When the GOS#
#is discovered the function that pertains to that      #
#particular guest will be run.                         #
########################################################
if [ -f /etc/redhat-release ]; then
        if [ `grep -l "CentOS" /etc/redhat-release` ]; then
                ven="`awk '{print $1}' /etc/redhat-release`";
        else
                ven="`awk '{print $1 $2}' /etc/redhat-release`";
                osver="`awk '{print $7}' /etc/redhat-release`";
       fi

elif [ -f /etc/SuSE-release ]; then
        ven="`head -1 /etc/SuSE-release | awk '{print $1}'`";
        osver="`head -1 /etc/SuSE-release | awk '{print $5}'`";
else
	echo "you may be using an unsupported linux distribution"
fi

case "$ven" in

CentOS)
	echo "CentOS is not a supported guest type"
	;;
RedHat)
	rver=`echo $osver | cut -f1 -d.`
	
	if [ $rver = 4 ]; then
		rhel4
	elif [ $rver = 5 ]; then
		rhel5
	else
		echo "Unable to determine your version of Redhat Linux";
		#trap 'trap_abort' HUP INT QUIT TERM;
	fi
	;;
SUSE)
	if [ $osver = 9 ]; then
		sles9
	elif [ $osver = 10 ]; then
		sles10
    elif [ $osver = 11 ]; then 
        sles11
	else
		echo "Unable to determine your version of SuSE Linux";
		#trap 'trap_abort' HUP INT QUIT TERM;
	fi
	;;
*)
	echo "The version of linux you are using may not be supported";
esac
